// Flutter imports:
import 'package:flutter/material.dart';

// Package imports:
import 'package:flutter_screenutil/flutter_screenutil.dart';

// Project imports:
import 'package:zego_uikit/src/components/audio_video/audio_video_view.dart';
import 'package:zego_uikit/src/components/audio_video_container/gallery/layout_gallery_last_item.dart';
import 'package:zego_uikit/src/services/services.dart';
import 'gallery/grid_layout_delegate.dart';
import 'layout.dart';

/// layout config of gallery
class ZegoLayoutGalleryConfig extends ZegoLayout {
  /// whether to display rounded corners and spacing between views
  bool addBorderRadiusAndSpacingBetweenView;

  ZegoLayoutGalleryConfig({this.addBorderRadiusAndSpacingBetweenView = true})
      : super.internal();
}

/// picture in picture layout
class ZegoLayoutGallery extends StatefulWidget {
  const ZegoLayoutGallery({
    Key? key,
    required this.localUser,
    required this.remoteUser,
    required this.layoutConfig,
    this.backgroundColor = const Color(0xff171821),
    this.foregroundBuilder,
    this.backgroundBuilder,
  }) : super(key: key);

  final ZegoUIKitUser localUser;
  final List<ZegoUIKitUser> remoteUser;
  final ZegoLayoutGalleryConfig layoutConfig;

  final Color backgroundColor;
  final AudioVideoViewForegroundBuilder? foregroundBuilder;
  final AudioVideoViewBackgroundBuilder? backgroundBuilder;

  @override
  State<ZegoLayoutGallery> createState() => _ZegoLayoutGalleryState();
}

class _ZegoLayoutGalleryState extends State<ZegoLayoutGallery> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    var users = [widget.localUser, ...widget.remoteUser];

    const maxCount = 8;
    List<ZegoUIKitUser> layoutUsers = [];
    List<ZegoUIKitUser> lastUsers = [];
    if (users.length > maxCount) {
      layoutUsers = users.sublist(0, maxCount - 1);
      lastUsers = users.sublist(maxCount - 1);
    } else {
      layoutUsers = users;
    }

    var layoutItems = layoutUsers.map((user) {
      var audioVideoView = ZegoAudioVideoView(
        key: ValueKey(user.id),
        user: user,
        backgroundBuilder: widget.backgroundBuilder,
        foregroundBuilder: widget.foregroundBuilder,
        borderRadius: widget.layoutConfig.addBorderRadiusAndSpacingBetweenView
            ? 18.0.w
            : null,
        borderColor: Colors.transparent,
      );
      return LayoutId(
        id: user.id,
        child: audioVideoView,
      );
    }).toList();
    if (lastUsers.isNotEmpty) {
      layoutItems.add(LayoutId(
        id: "sbs_last_users",
        child: ZegoLayoutGalleryLastItem(
          users: lastUsers,
          backgroundColor: const Color(0xff4A4B4D),
          borderRadius: widget.layoutConfig.addBorderRadiusAndSpacingBetweenView
              ? 18.0.w
              : null,
          borderColor: Colors.transparent,
        ),
      ));
    }

    return Container(
      color: widget.backgroundColor,
      child: CustomMultiChildLayout(
        delegate: GridLayoutDelegate.autoFill(
          autoFillItems: layoutItems,
          columnCount: layoutItems.length > 2 ? 2 : 1,
          lastRowAlignment: GridLayoutAlignment.start,
          itemPadding: widget.layoutConfig.addBorderRadiusAndSpacingBetweenView
              ? Size(10.0.r, 10.0.r)
              : const Size(0, 0),
        ),
        children: layoutItems,
      ),
    );
  }
}
