// Flutter imports:
import 'package:flutter/material.dart';

// Project imports:
import 'package:zego_uikit/src/components/audio_video/audio_video_view.dart';
import 'package:zego_uikit/src/services/services.dart';
import 'layout.dart';
import 'picture_in_picture/defines.dart';
import 'picture_in_picture/layout_pip_small_item.dart';
import 'picture_in_picture/layout_pip_small_item_list.dart';

/// layout config of picture in picture
class ZegoLayoutPictureInPictureConfig extends ZegoLayout {
  /// whether to hide the local View when the local camera is closed
  bool showMyViewWithVideoOnly;

  /// small video view is draggable if set true
  bool isSmallViewDraggable;

  ///  Whether you can switch view's position by clicking on the small view
  bool switchLargeOrSmallViewByClick;

  /// default position of small video view
  ZegoViewPosition smallViewPosition;

  /// would be local user if null
  String? largeViewDefaultUserID;

  ///
  Size? smallViewSize;

  ///
  EdgeInsets? smallViewSpacing;

  ZegoLayoutPictureInPictureConfig({
    this.showMyViewWithVideoOnly = false,
    this.isSmallViewDraggable = true,
    this.switchLargeOrSmallViewByClick = true,
    this.smallViewPosition = ZegoViewPosition.topRight,
    this.largeViewDefaultUserID,
    this.smallViewSize,
    this.smallViewSpacing,
  }) : super.internal();
}

/// picture in picture layout
class ZegoLayoutPictureInPicture extends StatefulWidget {
  const ZegoLayoutPictureInPicture({
    Key? key,
    required this.remoteUsers,
    required this.layoutConfig,
    this.foregroundBuilder,
    this.backgroundBuilder,
  }) : super(key: key);

  final List<ZegoUIKitUser> remoteUsers;
  final ZegoLayoutPictureInPictureConfig layoutConfig;

  final AudioVideoViewForegroundBuilder? foregroundBuilder;
  final AudioVideoViewBackgroundBuilder? backgroundBuilder;

  @override
  State<ZegoLayoutPictureInPicture> createState() =>
      _ZegoLayoutPictureInPictureState();
}

class _ZegoLayoutPictureInPictureState
    extends State<ZegoLayoutPictureInPicture> {
  ZegoUIKitUser? largeViewUser;
  List<ZegoUIKitUser> remoteUsers = [];

  late ZegoViewPosition currentPosition;

  @override
  void initState() {
    super.initState();

    currentPosition = widget.layoutConfig.smallViewPosition;
  }

  @override
  Widget build(BuildContext context) {
    remoteUsers = List.from(widget.remoteUsers);

    if (remoteUsers.length > 1) {
      return multiPIP();
    }

    return oneOnOnePIP();
  }

  Widget oneOnOnePIP() {
    var localUser = ZegoUIKit().getLocalUser();

    largeViewUser ??=
        ZegoUIKit().getUser(widget.layoutConfig.largeViewDefaultUserID ?? "");
    ZegoUIKitUser? smallViewUser = largeViewUser?.id == localUser.id
        ? (remoteUsers.isNotEmpty ? remoteUsers.first : null)
        : localUser;
    largeViewUser ??= remoteUsers.isNotEmpty ? remoteUsers.first : null;

    return Stack(
      children: [
        ZegoAudioVideoView(
          key: ValueKey(largeViewUser?.id),
          user: largeViewUser,
          backgroundBuilder: widget.backgroundBuilder,
          foregroundBuilder: widget.foregroundBuilder,
        ),
        ZegoLayoutPIPSmallItem(
          targetUser: smallViewUser,
          localUser: localUser,
          defaultPosition: widget.layoutConfig.smallViewPosition,
          draggable: widget.layoutConfig.isSmallViewDraggable,
          showLocalViewWithVideoOnly:
              widget.layoutConfig.showMyViewWithVideoOnly,
          onTap: (ZegoUIKitUser? user) {
            setState(() {
              if (!widget.layoutConfig.switchLargeOrSmallViewByClick) {
                return;
              }
              largeViewUser = user;
            });
          },
          size: widget.layoutConfig.smallViewSize,
          margin: widget.layoutConfig.smallViewSpacing,
          foregroundBuilder: widget.foregroundBuilder,
          backgroundBuilder: widget.backgroundBuilder,
        ),
      ],
    );
  }

  Widget multiPIP() {
    var localUser = ZegoUIKit().getLocalUser();

    largeViewUser ??= ZegoUIKit()
        .getUser(widget.layoutConfig.largeViewDefaultUserID ?? localUser.id);

    var indexOfLargeUserInRemoteList =
        remoteUsers.indexWhere((user) => largeViewUser?.id == user.id);
    if (-1 != indexOfLargeUserInRemoteList) {
      ///  large view user is an item of remote users, remove from remote
      remoteUsers.removeAt(indexOfLargeUserInRemoteList);
    }
    if (largeViewUser?.id != localUser.id) {
      /// local user is not in large view, put it to small view list
      remoteUsers.add(localUser);
    }

    return Stack(
      children: [
        ZegoAudioVideoView(
          key: ValueKey(largeViewUser?.id),
          user: largeViewUser,
          backgroundBuilder: widget.backgroundBuilder,
          foregroundBuilder: widget.foregroundBuilder,
        ),
        ZegoLayoutPIPSmallItemList(
          targetUsers: remoteUsers,
          localUser: ZegoUIKit().getLocalUser(),
          defaultPosition: widget.layoutConfig.smallViewPosition,
          showLocalViewWithVideoOnly:
              widget.layoutConfig.showMyViewWithVideoOnly,
          onTap: (ZegoUIKitUser user) {
            setState(() {
              if (!widget.layoutConfig.switchLargeOrSmallViewByClick) {
                return;
              }

              /// swap large view user to small view user
              if (largeViewUser != null) {
                remoteUsers.add(largeViewUser!);
              }
              remoteUsers.removeWhere((remoteUser) => user.id == remoteUser.id);
              largeViewUser = user;
            });
          },
          size: widget.layoutConfig.smallViewSize,
          margin: widget.layoutConfig.smallViewSpacing,
          foregroundBuilder: widget.foregroundBuilder,
          backgroundBuilder: widget.backgroundBuilder,
        ),
      ],
    );
  }
}
