// Dart imports:

// Flutter imports:
import 'package:flutter/cupertino.dart';

// Project imports:
import 'package:zego_uikit/zego_uikit.dart';

class ZegoUIKitInvitationService {
  static final ZegoUIKitInvitationService shared =
      ZegoUIKitInvitationService._internal();

  factory ZegoUIKitInvitationService() => shared;

  ZegoUIKitInvitationService._internal() {
    WidgetsFlutterBinding.ensureInitialized();
    assert(ZegoUIKit().getPlugin(ZegoUIKitPluginType.signaling) != null);
  }

  Future<void> init(int appID, {String appSign = ''}) async {
    await ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .invoke('init', {'appID': appID, 'appSign': appSign});
  }

  Future<void> uninit() async {
    await ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .invoke('uninit', {});
  }

  Future<void> login(String id, String name) async {
    await ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .invoke('login', {'userID': id, 'userName': name});
  }

  Future<void> logout() async {
    await ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .invoke('logout', {});
  }

  /// send invitation to one or more specified users
  /// [invitees] list of invitees.
  /// [timeout]timeout of the call invitation, the unit is seconds
  /// [type] call type
  /// [data] extended field, through which the inviter can carry information to the invitee.
  Future<List<String>> sendInvitation(
    String inviterName,
    List<String> invitees,
    int timeout,
    int type,
    String data,
  ) async {
    Map result = await ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .invoke('sendInvitation', {
      'inviterName': inviterName,
      'invitees': invitees,
      'timeout': timeout,
      'type': type,
      'data': data
    });

    return result['errorInvitees'] as List<String>;
  }

  /// cancel invitation to one or more specified userze gs
  /// [inviteeID] invitee's id
  /// [data] extended field
  Future<List<String>> cancelInvitation(
      List<String> invitees, String data) async {
    Map result = await ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .invoke('cancelInvitation', {'invitees': invitees, 'data': data});

    return result['errorInvitees'] as List<String>;
  }

  Future<void> refuseInvitation(String inviterID, String data) async {
    await ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .invoke('refuseInvitation', {'inviterID': inviterID, 'data': data});
  }

  /// invitee accept the call invitation
  /// [inviterID] inviter id, who send invitation
  /// [data] extended field
  Future<void> acceptInvitation(String inviterID, String data) async {
    await ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .invoke('acceptInvitation', {'inviterID': inviterID, 'data': data});
  }

  /// stream callback, notify invitee when call invitation received
  Stream<Map> getInvitationConnectionStateStream() {
    return ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .getEventStream('invitationConnectionState');
  }

  /// stream callback, notify invitee when call invitation received
  Stream<Map> getInvitationReceivedStream() {
    return ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .getEventStream('invitationReceived');
  }

  /// stream callback, notify invitee if invitation timeout
  Stream<Map> getInvitationTimeoutStream() {
    return ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .getEventStream('invitationTimeout');
  }

  /// stream callback, When the call invitation times out, the invitee does not respond, and the inviter will receive a callback.
  Stream<Map> getInvitationResponseTimeoutStream() {
    return ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .getEventStream('invitationResponseTimeout');
  }

  /// stream callback, notify when call invitation accepted by invitee
  Stream<Map> getInvitationAcceptedStream() {
    return ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .getEventStream('invitationAccepted');
  }

  /// stream callback, notify when call invitation rejected by invitee
  Stream<Map> getInvitationRefusedStream() {
    return ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .getEventStream('invitationRefused');
  }

  /// stream callback, notify when call invitation cancelled by inviter
  Stream<Map> getInvitationCanceledStream() {
    return ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .getEventStream('invitationCanceled');
  }
}
